<?php

namespace App\Http\Controllers\Admin;

use App\ClothService;
use App\Http\Requests\AdminStoreClothServiceRequest;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;

class AdminClothServiceController extends Controller
{
    public function index()
    {
        $items = ClothService::orderBy('created_at', 'desc')->paginate(15);
        $title = "قطع الملابس ";
        return view('clothesServices', compact('title', 'items'))->with(['controller' => $this]);
    }

    public function show($id)
    {
        $item = ClothService::find($id);
        if (!$item) {
            return view('errors.404');
        }
        $title = "بيانات قطعة الملابس";
        return view('clothService', compact('title', 'item'))->with(['controller' => $this]);
    }

    public function store(AdminStoreClothServiceRequest $request)
    {
        try {
            ClothService::create($request->all());
            return back()->with('success', 'تم عملية الإضافة بنجاح..');
        } catch (\Exception $exception) {
            return back()->with('success', 'حدث خلل غير متوقع أثناء عملية الإضافة');
        }
    }

    public function update(AdminStoreClothServiceRequest $request, $id)
    {
        try {
            ClothService::find($id)->update(["name" => $request->name]);
            return back()->with('success', 'تم عملية التحديث بنجاح..');
        } catch (\Exception $exception) {
            return back()->with('success', 'حدث خلل غير متوقع أثناء عملية تحديث البيانات');
        }
    }

    public function delete(Request $request)
    {
        if (!$request->id) {
            return 'معرف قطعة الملابس غير صالح';
        }
        $item = ClothService::find($request->id);
        if ($item != null) {
            if ($item->delete()) {
                return "تمت عملية الحذف بنجاح";
            } else {
                return "حدث خطأ غير متوقع أثناء الحذف .. حاول في وقت آخر !";
            }
        } else {
            return " تحاول حذف عنصر غير موجود ..  !";
        }
    }
}
