<?php

namespace App\Http\Controllers\Admin;

use App\Subscription;
use App\SubscriptionOrder;
use App\User;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;

class AdminSubscriptionOrderController extends Controller
{
    public function index()
    {
        $items = SubscriptionOrder::where('is_accepted', '=', -1)->orderBy('created_at', 'desc')->paginate(15);
        $title = "طلبات إشتراك المغاسل الجديدة ";
        return view('subscriptionOrders', compact('items', 'title'))->with(['controller' => $this]);
    }

    public function show($id)
    {
        $item = SubscriptionOrder::where('laundry_id', '=', $id)->first();
        if (!$item) {
            return view('errors.404');
        }
        $title = "طلب إشتراك";
        return view('subscriptionOrder', compact('item', 'title'))->with(['controller' => $this]);
    }

    public function accepted()
    {
        $title = "طلبات إشتراك المغاسل المقبولة ";
        $items = SubscriptionOrder::where('is_accepted', '=', 1)->orderBy('created_at', 'desc')->paginate('15');
        return view('subscriptionOrders', compact('items', 'title'))->with(['controller' => $this]);
    }

    public function rejected()
    {
        $title = "طلبات إشتراك المغاسل المرفوضة";
        $items = SubscriptionOrder::where('is_accepted', '=', 0)->orderBy('created_at', 'desc')->paginate('15');
        return view('subscriptionOrders', compact('items', 'title'))->with(['controller' => $this]);
    }

    public function accept($id, $duration)
    {
        $accepted = SubscriptionOrder::find($id);
        $accepted->is_accepted = 1;
        if ($accepted->save()) {
            //verify laundry
            $user = User::find($accepted->laundry_id);
            $user->verification->is_verified = 1;
            if (!($user->verification->save())) {
                $accepted->is_accepted = 0;
                return back()->with('success', 'حدث خطأ غير متوقع : أثناء إضافة الإشتراك..');
            }
            //add subscription
            $subscription = new Subscription();
            $subscription->subscription_plan_id = $accepted->subscriptionPlan->id;
            $subscription->subscription_order_id = $accepted->id;
            //count expired date, get duration from selected plan
            $timestamp = time();
            $timestamp += 60 * 60 * 24 * $duration;
            $subscription->expired_at = date('Y-m-d H:i:s', $timestamp);
            if (!($user->subscription()->save($subscription))) {
                $accepted->is_accepted = 0;
                $user->verification->is_verified = 0;
                return back()->with('success', 'حدث خطأ غير متوقع : أثناء إضافة الإشتراك..');
            }
            return back()->with('success', 'تم إضافة الإشتراك بنجاح..');
        } else {
            return back()->with('success', 'حدث خلل أثناء إضافة الإشتراك');
        }
    }

    public function reject($id)
    {
        $rejected = SubscriptionOrder::find($id);
        $rejected->is_accepted = 0;
        if ($rejected->save()) {
            return back()->with('success', 'تم رفض  طلب الإشتراك بنجاح..');
        } else {
            return back()->with('success', 'حدث خلل أثناء  رفض طلب الإشتراك');
        }
    }

    public function delete(Request $request)
    {
        if (!$request->id) {
            return 'معرف الرسالة غير صالح';
        }
        $item = SubscriptionOrder::find($request->id);
        if ($item != null) {
            if ($item->delete()) {
                return "تمت عملية الحذف بنجاح";
            } else {
                return "حدث خطأ غير متوقع أثناء الحذف .. حاول في وقت آخر !";
            }
        } else {
            return " تحاول حذف عنصر غير موجود ..  !";
        }
    }
}
