<?php

namespace App\Http\Controllers\Api;

use App\AdditionalService;
use App\LaundryAdditionalPrice;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Validator;

class ApiLaundryAdditionalPriceController extends Controller
{
    public function store(Request $request, $id)
    {
        try {
            $user = Auth::guard('api')->user();
            if ($user->role != 2) {
                return response()->json($this->result(false, "هذه الخدمة فقط للمغاسل ", -4));
            }
            if ($this->isSubscriptionExpired($user)) {
                $user->api_token = null;
                $user->save();
                return response()->json($this->result(false, "للأسف الإشتراك الحالي منتهي ولا يمكنك تسجيل الدخول، يرجى تجديد الإشتراك", -6));
            }
            //request validation
            $validator = Validator::make($request->all(), [
                'washing' => ['required', 'numeric'],
            ]);
            if ($validator->fails()) {
                return response()->json($this->resultWithVar(false, "يجب إدخال جميع البيانات بالشكل الصحيح", 0, "errors", $this->validatorErrorsToStringArray($validator)));
            }
            //check if id belong to additional service
            $additional_service = AdditionalService::find($id);
            if (!$additional_service) {
                return response()->json($this->result(false, "تعذر الحصول على الخدمة الإضافية المراد إضافتها", -3));
            }
            //check if additional service price in current laundry additional services
            $additional_service_price = LaundryAdditionalPrice::where('laundry_id', '=', $user->id)->where('name', '=', $additional_service->name)->first();
            if (!$additional_service_price) {
                //store
                $item = new LaundryAdditionalPrice();
                $item->name = $additional_service->name;
                $item->unit = $additional_service->unit;
                $item->units = $additional_service->units;
                $item->washing = $request->washing;
                if ($user->laundryAdditionalPrices()->save($item)) {
                    return response()->json($this->resultWithVar(true, "تم إضافة خدمة إضافية جديدة للمغسلة بنجاح", 1, "additional_service_price", $item));
                } else {
                    return response()->json($this->result(false, "حدث خلل غير متوقع أثناء الإرسال، حاول في وقت آخر ", -2));
                }
            } else {
                //update
                $additional_service_price->washing = $request->washing;
                if ($additional_service_price->save()) {
                    /*                    $updated_additional_service_price = LaundryAdditionalPrice::with('additionalService')->find($additional_service_price);*/
                    return response()->json($this->resultWithVar(true, "تم تحديث أسعار الخدمة بنجاح", 3, "additional_service_price", $additional_service_price));
                } else {
                    return response()->json($this->result(false, "حدث خلل غير متوقع أثناء الإرسال، حاول في وقت آخر ", -2));
                }
            }
        } catch (\Exception $exception) {
            return response()->json($this->result(false, "حدث خلل غير متوقع أثناء الإرسال، حاول في وقت آخر ", -2));
        }
    }

    public function delete($id)
    {
        try {
            $user = Auth::guard('api')->user();
            if ($user->role != 2) {
                return response()->json($this->result(false, "هذه الخدمة فقط للمغاسل ", -4));
            }

            if ($this->isSubscriptionExpired($user)) {
                $user->api_token = null;
                $user->save();
                return response()->json($this->result(false, "للأسف الإشتراك الحالي منتهي ولا يمكنك تسجيل الدخول، يرجى تجديد الإشتراك", -6));
            }
            $item = LaundryAdditionalPrice::where('id', '=', $id)->where('laundry_id', '=', $user->id)->first();
            if (!$item) {
                return response()->json($this->result(false, "تعذر الحصول على الخدمة الإضافية المراد حذفها", -3));
            }
            if (!($item->delete())) {
                return response()->json($this->result(false, "حدث خلل غير متوقع أثناء الإرسال، حاول في وقت آخر ", -2));
            }
            return response()->json($this->result(true, "تم حذف الخدمة الإضافية بنجاح", 1));
        } catch (\Exception $exception) {
            return response()->json($this->result(false, "حدث خلل غير متوقع أثناء الإرسال، حاول في وقت آخر ", -2));
        }
    }
}
