<?php

namespace App\Http\Controllers\Api;

use App\ClothService;
use App\LaundryClothPrice;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Validator;

class ApiLaundryClothPriceController extends Controller
{
    public function store(Request $request)
    {
        try {
            $user = Auth::guard('api')->user();
            if ($user->role != 2) {
                return response()->json($this->result(false, "هذه الخدمة فقط للمغاسل ", -4));
            }

            if ($this->isSubscriptionExpired($user)) {
                $user->api_token = null;
                $user->save();
                return response()->json($this->result(false, "للأسف الإشتراك الحالي منتهي ولا يمكنك تسجيل الدخول، يرجى تجديد الإشتراك", -6));
            }

            //request validation
            $validator = Validator::make($request->all(), [
                'name' => ['required', 'string'],
                'white_washing' => ['required', 'numeric'],
                'colored_washing' => ['required', 'numeric'],
                'ironing' => ['required', 'numeric'],
            ]);
            if ($validator->fails()) {
                return response()->json($this->resultWithVar(false, "يجب إدخال جميع البيانات بالشكل الصحيح", 0, "errors", $this->validatorErrorsToStringArray($validator)));
            }
            //check if cloth service price in current laundry clothes services
            $cloth_service_price = LaundryClothPrice::where('laundry_id', '=', $user->id)->where('name', '=', $request->name)->first();
            if (!$cloth_service_price) {
                //store
                $item = new LaundryClothPrice();
                $item->name = $request->name;
                $item->white_washing = $request->white_washing;
                $item->colored_washing = $request->colored_washing;
                $item->ironing = $request->ironing;
                if ($user->laundryClothesPrices()->save($item)) {
                    /*                    $inserted_item = LaundryClothPrice::with('clothService')->find($item->id);*/
                    return response()->json($this->resultWithVar(true, "تم إضافة خدمة ملابس جديدة للمغسلة بنجاح", 1, "cloth_service_price", $item));
                } else {
                    return response()->json($this->result(false, "حدث خلل غير متوقع أثناء الإرسال، حاول في وقت آخر ", -2));
                }
            } else {
                //update
                $cloth_service_price->white_washing = $request->white_washing;
                $cloth_service_price->colored_washing = $request->colored_washing;
                $cloth_service_price->ironing = $request->ironing;
                if ($cloth_service_price->save()) {
                    /*                    $updated_cloth_service_price = LaundryClothPrice::with('clothService')->find($cloth_service_price);*/
                    return response()->json($this->resultWithVar(true, "تم تحديث أسعار الخدمة بنجاح", 3, "cloth_service_price", $cloth_service_price));
                } else {
                    return response()->json($this->result(false, "حدث خلل غير متوقع أثناء الإرسال، حاول في وقت آخر ", -2));
                }
            }
        } catch (\Exception $exception) {
            return response()->json($this->result(false, "حدث خلل غير متوقع أثناء الإرسال، حاول في وقت آخر ", -2));
        }
    }

    public function delete($id)
    {
        try {
            $user = Auth::guard('api')->user();
            if ($user->role != 2) {
                return response()->json($this->result(false, "هذه الخدمة فقط للمغاسل ", -4));
            }

            if ($this->isSubscriptionExpired($user)) {
                $user->api_token = null;
                $user->save();
                return response()->json($this->result(false, "للأسف الإشتراك الحالي منتهي ولا يمكنك تسجيل الدخول، يرجى تجديد الإشتراك", -6));
            }
            $item = LaundryClothPrice::where('id', '=', $id)->where('laundry_id', '=', $user->id)->first();
            if (!$item) {
                return response()->json($this->result(false, "تعذر الحصول على خدمة الملابس المراد حذفها", -3));
            }
            if (!($item->delete())) {
                return response()->json($this->result(false, "حدث خلل غير متوقع أثناء الإرسال، حاول في وقت آخر ", -2));
            }
            return response()->json($this->result(true, "تم حذف خدمة الملابس بنجاح", 1));
        } catch (\Exception $exception) {
            return response()->json($this->result(false, "حدث خلل غير متوقع أثناء الإرسال، حاول في وقت آخر ", -2));
        }
    }
}
