<?php

namespace App\Http\Controllers\Api;

use App\Location;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Validator;
use Symfony\Component\HttpKernel\Client;

class ApiLocationController extends Controller
{
    public function nearby(Request $request)
    {
        try {
            //detect user
            $user = Auth::guard('api')->user();
            //check if user is a customer
            if ($user->role != 3) {
                return response()->json($this->result(false, "هذه الخدمة فقط للعملاء ", -4));
            }
            //request validation
            $validator = Validator::make($request->all(), [
                'lat' => 'required', 'numeric',
                'lng' => 'required', 'numeric',
            ]);
            if ($validator->fails()) {
                return response()->json($this->resultWithVar(false, "يجب إدخال جميع البيانات بالشكل الصحيح", 0, "errors", $this->validatorErrorsToStringArray($validator)));
            }
            $request_lat = $request->lat;
            $request_lng = $request->lng;
            $timestamp = time();
            $items = DB::table('locations')
                ->join('users', 'users.id', '=', 'locations.user_id')
                ->join('users_profiles', 'users_profiles.user_id', '=', 'users.id')
                ->join('verifications', 'verifications.user_id', '=', 'users.id')
                ->where('users.role', '=', 2)
                ->where('users_profiles.isActive', '=', 1)
                ->where('verifications.is_verified', '=', 1)
                ->select(DB::raw('locations.id'))
                ->get();
            if ($items->count() == 0) {
                return response()->json($this->resultWithVar(true, "تم جلب المغاسل القريبة بحدود 50 كم من الموقع المحدد بنجاح", 1, "laundries", $items));
            }

            $ids = [];
            foreach ($items as $item){
                array_push($ids, $item->id);
            }

            $items = DB::table('locations')
                ->join('users', 'users.id', '=', 'locations.user_id')
                ->join('users_profiles', 'users_profiles.user_id', '=', 'users.id')
                ->join('subscriptions', 'subscriptions.laundry_id', '=', 'users.id')
                ->whereIn('locations.id', $ids)
                ->where('subscriptions.expired_at', '>', date('Y-m-d H:i:s', $timestamp))
                ->select(DB::raw('users.id, users_profiles.name, users.phone, users_profiles.photo, locations.lat, locations.lng, ( 6371 * acos( cos( radians(' . $request_lat . ') ) * cos( radians( locations.lat ) ) * cos( radians( locations.lng ) - radians(' . $request_lng . ') ) + sin( radians(' . $request_lat . ') ) * sin( radians(locations.lat) ) ) ) AS time'))
                ->having('time', '<', 50)
                ->take(10)
                ->get();
            $times = [];

            foreach ($items as $i){
/*                preg_match("/([0-9]+)/", $this->GetDrivingDistance($request->lat, $i->lat, $request->lng, $i->lng), $matches);*/
/*                filter_var($this->GetDrivingDistance($request->lat, $i->lat, $request->lng, $i->lng), FILTER_SANITIZE_NUMBER_INT)*/
               array_push($times, ceil($this->GetDrivingDistance($request->lat, $i->lat, $request->lng, $i->lng)/60));
            }
            $count= 0;
            foreach ($items as $i){
                $i->time = $times[$count];
                $count++;
            }
          /*  $client = new \GuzzleHttp\Client();*/

            /*$response = $client->post('https://www.google.com/recaptcha/api/siteverify', [
                'form_params' => array(
                    'secret' => '6LdZPzcUAAAAAAXFI9yvbAVfrMfCWa3xWLGcFFtx',
                    'response' => $reCaptcha_token
                )
            ]);*/
            //check if laundries subscription expired
            //if subscription expired set laundry verification is_verified to 0

            return response()->json($this->resultWithVar(true, "تم جلب المغاسل القريبة بحدود 50 كم من الموقع المحدد بنجاح", 1, "laundries", $items));
        } catch (\Exception $exception) {
            return response()->json($this->result(false, "حدث خلل غير متوقع أثناء الإرسال، حاول في وقت آخر ", -2));
        }
    }

    function GetDrivingDistance($lat1, $lat2, $long1, $long2)
    {
        $url = "https://maps.googleapis.com/maps/api/distancematrix/json?key=AIzaSyD08gZlL4TtCxWjUgrq9UsDzKCeqXJzo20&origins=".$lat1.",".$long1."&destinations=".$lat2.",".$long2."&mode=driving&language=ar";
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_PROXYPORT, 3128);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
        $response = curl_exec($ch);
        curl_close($ch);
        $response_a = json_decode($response, true);
/*        $dist = $response_a['rows'][0]['elements'][0]['distance']['text'];*/
        $time = $response_a['rows'][0]['elements'][0]['duration']['value'];

        return /*array('distance' => $dist, 'time' => */$time/*)*/;
    }
}
