<?php

namespace App\Http\Controllers\Api;

use App\LaundryAdditionalPrice;
use App\LaundryClothPrice;
use App\Order;
use App\OrderAdditionalService;
use App\OrderCloth;
use App\User;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Input;
use Illuminate\Support\Facades\Validator;
use function MongoDB\BSON\toJSON;
use Psy\Util\Json;

class ApiOrderController extends Controller
{
    public function index($status)
    {
        try {
            $status_array = array(-1, 0, 1);
            if (!in_array($status, $status_array)) {
                return response()->json($this->result(false, "تعذر الحصول على الطلبات، تأكد من حالة الطلبات المراد البحث عنها", -3));
            }
            $user = Auth::guard('api')->user();
            if ($user->role == 2) {
                if ($this->isSubscriptionExpired($user)) {
                    $user->api_token = null;
                    $user->save();
                    return response()->json($this->result(false, "للأسف الإشتراك الحالي منتهي ولا يمكنك تسجيل الدخول، يرجى تجديد الإشتراك", -6));
                }
                //for laundry
                $items = Order::with(['laundry', 'laundry.profile', 'laundry.location', 'customer', 'customer.profile', 'customer.location', 'OrderClothes', 'orderAdditionalServices', 'orderAdditionalServices'])->where('laundry_id', '=', $user->id)->where('status', '=', $status)->get();

            } elseif ($user->role == 3) {
                $customer_id = $user->id;
                //for laundry
                $items = Order::with(['customer', 'customer.profile', 'customer.location', 'laundry', 'laundry.profile', 'laundry.location', 'laundry.has_rate' => function ($q) use ($customer_id/*, $laundry_id*/) {
                    $q->where('user_id', '=', $customer_id)/*->where('laundry_id', '=', $laundry_id)*/
                    ;
                }, 'OrderClothes', 'orderAdditionalServices', 'orderAdditionalServices'])->where('customer_id', '=', $user->id)->where('status', '=', $status)->get();
            }
            return response()->json($this->resultWithVar(true, "تم جلب الطلبات بنجاح", 1, "orders", $items));
        } catch (\Exception $exception) {
            return response()->json($this->result(false, "حدث خلل غير متوقع أثناء الإرسال، حاول في وقت آخر ", -2));
        }
    }

    public function store(Request $request)
    {
        try {
            $user = Auth::guard('api')->user();
            //check if user is a customer
            if ($user->role != 3) {
                return response()->json($this->result(false, "هذه الخدمة فقط للعملاء ", -4));
            }
            $decode_json = json_decode($request->getContent(false), true);
            $clothes_items = $decode_json['clothes_items'];
            $additional_items = $decode_json['additional_items'];

            //check if laundry exist and not expired
            try {
                $decode_json['laundry_id'];
            } catch (\Exception $exception) {
                return response()->json($this->result(false, "يجب إدخال جميع البيانات بالشكل الصحيح", 0));
            }
            $laundry = User::where('id', '=', $decode_json['laundry_id'])->where('role', '=', 2)->first();
            if (!$laundry) {
                return response()->json($this->result(false, "تعذر الحصول على المغسلة", -3));
            }
            //check if laundry not expired
            if ($this->isSubscriptionExpired($laundry)) {
                return response()->json($this->result(false, "تعذر الحصول على المغسلة", -3));
            }
            //store order
            $order = new Order();
            try {
                $order->laundry_id = $decode_json['laundry_id'];
                $order->customer_id = $user->id;
                $order->lat = $decode_json['lat'];
                $order->lng = $decode_json['lng'];
                $order->customerAddress = $this->GetAddress($decode_json['lat'], $decode_json['lat'], $decode_json['lng'], $decode_json['lng']);
                $order->cost = $decode_json['cost'];
                $order->save();
            } catch (\Exception $exception) {
                return response()->json($this->result(false, "يجب إدخال جميع البيانات بالشكل الصحيح", 0));
            }
            $clothes_count = 0;
            foreach ($clothes_items as $c) {
                $clothes_count++;
            }

            $additional_count = 0;
            foreach ($additional_items as $c) {
                $additional_count++;
            }

            if ($clothes_count == 0 and $additional_count == 0) {
                return response()->json($this->result(false, "يجب اضافة عنصر واحد على الأقل جميع البيانات بالشكل الصحيح", 0));
            }
            if ($clothes_count != 0) {
                foreach ($clothes_items as $cloth_item) {
                    $order_cloth = new OrderCloth();
                    $order_cloth->order_id = $order->id;
                    $laundry_cloth_price = LaundryClothPrice::find($cloth_item["laundry_cloth_price_id"]);
                    if (!$laundry_cloth_price) {
                        $order->delete();
                        return response()->json($this->result(false, "حدث خلل غير متوقع أثناء الإرسال، حاول في وقت آخر ", -2));
                    }
                    $order_cloth->laundry_cloth_price_id = $laundry_cloth_price->id;
                    $order_cloth->cloth_name = $laundry_cloth_price->name;
                    $order_cloth->white_count = $cloth_item["white_count"];
                    $order_cloth->colored_count = $cloth_item["colored_count"];
                    $order_cloth->washingOrIroning = $cloth_item["washingOrIroning"];
                    if (!$order_cloth->save()) {
                        $order->delete();
                        return response()->json($this->result(false, "حدث خلل غير متوقع أثناء الإرسال، حاول في وقت آخر ", -2));
                    }
                }
            }
            if ($additional_count != 0) {
                foreach ($additional_items as $additional_item) {
                    $order_additional_service = new OrderAdditionalService();
                    $order_additional_service->order_id = $order->id;
                    $laundry_additional_price = LaundryAdditionalPrice::find($additional_item["laundry_additional_price_id"]);
                    if (!$laundry_additional_price) {
                        $order->delete();
                        return response()->json($this->result(false, "حدث خلل غير متوقع أثناء الإرسال، حاول في وقت آخر ", -2));
                    }
                    $order_additional_service->laundry_additional_price_id = $laundry_additional_price->id;
                    $order_additional_service->additional_service_name = $laundry_additional_price->name;
                    $order_additional_service->unit = $laundry_additional_price->unit;
                    $order_additional_service->units = $laundry_additional_price->units;
                    $order_additional_service->count = $additional_item["count"];
                    if (!$order_additional_service->save()) {
                        $order->delete();
                        return response()->json($this->result(false, "حدث خلل غير متوقع أثناء الإرسال، حاول في وقت آخر ", -2));
                    }
                }
                if ($order->orderAdditionalServices->count() != $additional_count or $order->OrderClothes->count() != $clothes_count) {
                    $order->delete();
                    return response()->json($this->result(false, "حدث خلل غير متوقع أثناء الإرسال، حاول في وقت آخر ", -2));
                }
            }
            $customer_id = $user->id;
            $item = Order::with(['customer', 'customer.profile', 'customer.location', 'laundry', 'laundry.profile', 'laundry.location', 'laundry.has_rate' => function ($q) use ($customer_id/*, $laundry_id*/) {
                $q->where('user_id', '=', $customer_id)/*->where('laundry_id', '=', $laundry_id)*/
                ;
            }, 'OrderClothes', 'orderAdditionalServices', 'orderAdditionalServices'])->find($order->id);
            if (!$item) {
                return response()->json($this->result(false, "حدث خلل غير متوقع أثناء الإرسال، حاول في وقت آخر ", -2));
            }
            return response()->json($this->resultWithVar(true, "تم إضافة الطلب بنجاح", 1, "order", $item));
        } catch (\Exception $exception) {
            return response()->json($this->result(false, "حدث خلل غير متوقع أثناء الإرسال، حاول في وقت آخر ", -2));
        }

    }

    public function updateItems(Request $request)
    {
        try {
            $user = Auth::guard('api')->user();
            //check if user is a customer
            if ($user->role != 3) {
                return response()->json($this->result(false, "هذه الخدمة فقط للعملاء ", -4));
            }
            $decode_json = json_decode($request->getContent(false), true);
            $clothes_items = $decode_json['clothes_items'];
            $additional_items = $decode_json['additional_items'];


            $clothes_count = 0;
            foreach ($clothes_items as $c) {
                $clothes_count++;
            }

            $additional_count = 0;
            foreach ($additional_items as $c) {
                $additional_count++;
            }

            if ($clothes_count == 0 and $additional_count == 0) {
                return response()->json($this->result(false, "لا يوجد عناصر لتعديلها في الطلب المرسل", 0));
            }
            if ($clothes_count != 0) {
                foreach ($clothes_items as $cloth_item) {
                    try {
                        //detect order_id
                        $cloth_item_object = OrderCloth::find($cloth_item['id']);
                        //check if element found
                        if ($cloth_item_object) {
                            //check if order_cloth related to customer
                            $order = Order::where('id', '=', $cloth_item_object->order_id)->where('customer_id', '=', $user->id)->first();
                            if (!$order) {
                                return response()->json($this->result(false, "تعذر الحصول على العنصر المراد تحديثه", -3));
                            }
                            $item_with_status = Order::where('id', '=', $cloth_item_object->order_id)->where('customer_id', '=', $user->id)->where('status', '=', -1)->first();
                            if (!$item_with_status) {
                                return response()->json($this->result(false, "تعذر تحديث عنصر من الطلب، فقط يمكنك تحديث عنصر من الطلب الجديد", -3));
                            }
                            //update item counts and order cost
                            //update item counts
                            $current_item_cost = ($this->getClothWhiteCost($cloth_item_object->laundry_cloth_price_id, $cloth_item_object->washingOrIroning, 0) * $cloth_item_object->white_count) + ($this->getClothColoredCost($cloth_item_object->laundry_cloth_price_id, $cloth_item_object->washingOrIroning, 0) * $cloth_item_object->colored_count);
                            $cloth_item_object->white_count = $cloth_item['white_count'];
                            $cloth_item_object->colored_count = $cloth_item['colored_count'];
                            if ($cloth_item_object->save()) {
                                //update order cost
                                $order->cost = $order->cost - $current_item_cost;
                                $order->save();
                                $order->cost = $order->cost + ($this->getClothWhiteCost($cloth_item_object->laundry_cloth_price_id, $cloth_item_object->washingOrIroning, 0) * $cloth_item['white_count']) + ($this->getClothColoredCost($cloth_item_object->laundry_cloth_price_id, $cloth_item_object->washingOrIroning, 0) * $cloth_item['colored_count']);
                                $order->save();
                            }
                        }
                    } catch (\Exception $exception) {
                        return response()->json($this->result(false, "يجب إدخال جميع البيانات بالشكل الصحيح", 0));
                    }
                }
            }
            if ($additional_count != 0) {
                foreach ($additional_items as $additional_item) {
                    try {
                        //detect order_id
                        $additional_item_object = OrderAdditionalService::find($additional_item['id']);
                        //check if element found
                        if ($additional_item_object) {
                            //check if order_cloth related to customer
                            $order = Order::where('id', '=', $additional_item_object->order_id)->where('customer_id', '=', $user->id)->first();
                            if (!$order) {
                                return response()->json($this->result(false, "تعذر الحصول على العنصر المراد تحديثه", -3));
                            }
                            $item_with_status = Order::where('id', '=', $additional_item_object->order_id)->where('customer_id', '=', $user->id)->where('status', '=', -1)->first();
                            if (!$item_with_status) {
                                return response()->json($this->result(false, "تعذر تحديث عنصر من الطلب، فقط يمكنك تحديث عنصر من الطلب الجديد", -3));
                            }
                            //update item counts and order cost
                            //update item counts
                            $current_item_cost_one_count = $additional_item_object->laundryAdditionalPrice->washing;
                            $current_item_cost = $current_item_cost_one_count * $additional_item_object->count;
                            $additional_item_object->count = $additional_item['count'];
                            if ($additional_item_object->save()) {
                                //update order cost
                                $order->cost = $order->cost - $current_item_cost;
                                $order->save();
                                $order->cost = $order->cost + ($current_item_cost_one_count * $additional_item['count']);
                                $order->save();
                            }
                        }
                    } catch (\Exception $exception) {
                        return response()->json($this->result(false, "يجب إدخال جميع البيانات بالشكل الصحيح", 0));
                    }
                }
            }
            $customer_id = $user->id;
            $item = Order::with(['customer', 'customer.profile', 'customer.location', 'laundry', 'laundry.profile', 'laundry.location', 'laundry.has_rate' => function ($q) use ($customer_id/*, $laundry_id*/) {
                $q->where('user_id', '=', $customer_id)/*->where('laundry_id', '=', $laundry_id)*/
                ;
            }, 'OrderClothes', 'orderAdditionalServices', 'orderAdditionalServices'])->find($order->id);
            if (!$item) {
                return response()->json($this->result(false, "حدث خلل غير متوقع أثناء الإرسال، حاول في وقت آخر ", -2));
            }
            return response()->json($this->resultWithVar(true, "تم تعديل الطلب بنجاح", 1, "order", $item));
        } catch (\Exception $exception) {
            return response()->json($this->result(false, "حدث خلل غير متوقع أثناء الإرسال، حاول في وقت آخر ", -2));
        }

    }

    public function delete($id)
    {
        try {
            $user = Auth::guard('api')->user();
            if ($user->role != 3) {
                return response()->json($this->result(false, "هذه الخدمة فقط للعملاء ", -4));
            }

            $item = Order::where('id', '=', $id)->where('customer_id', '=', $user->id)->first();
            if (!$item) {
                return response()->json($this->result(false, "تعذر الحصول على الطلب المراد حذفه", -3));
            }
            $item_with_status = Order::where('id', '=', $id)->where('customer_id', '=', $user->id)->where('status', '=', -1)->first();
            if (!$item_with_status) {
                return response()->json($this->result(false, "تعذر حذف الطلب، فقط يمكنك حذف الطلب الجديد", -3));
            }
            if (!($item->delete())) {
                return response()->json($this->result(false, "حدث خلل غير متوقع أثناء الإرسال، حاول في وقت آخر ", -2));
            }
            return response()->json($this->result(true, "تم حذف الطلب بنجاح", 1));
        } catch (\Exception $exception) {
            return response()->json($this->result(false, "حدث خلل غير متوقع أثناء الإرسال، حاول في وقت آخر ", -2));
        }
    }

    public function updateStatus(Request $request, $id)
    {
        try {
            $user = Auth::guard('api')->user();
            if ($user->role != 2) {
                return response()->json($this->result(false, "هذه الخدمة فقط للمغاسل ", -4));
            }
            //request validation
            $validator = Validator::make($request->all(), [
                'status' => ['required', 'numeric', 'regex:/(-1|0|1)/'],
            ]);
            if ($validator->fails()) {
                return response()->json($this->resultWithVar(false, "يجب إدخال جميع البيانات بالشكل الصحيح", 0, "errors", $this->validatorErrorsToStringArray($validator)));
            }
            //check if order is exist and related to laundry
            $order = Order::where('id', '=', $id)->where('laundry_id', '=', $user->id)->first();
            if (!$order) {
                return response()->json($this->result(false, "تعذر الحصول على الطلب المراد تحديثه", -3));
            }

            //update status
            $order->status = $request->input('status');
            if (!($order->save())) {
                return response()->json($this->result(false, "حدث خلل غير متوقع أثناء الإرسال، حاول في وقت آخر ", -2));
            }

            //saved successfully
            $status = "تحديث";
            if($request->status == 0){
                $status = "قبول";
            }elseif ($request->status == 1){
                $status = "تسليم";
            }
            //return accepted if status  = 0
            $updated_order = Order::with(['laundry', 'laundry.profile', 'laundry.location', 'customer', 'customer.profile', 'customer.location', 'OrderClothes', 'orderAdditionalServices', 'orderAdditionalServices'])->find($order->id);
            return response()->json($this->resultWithVar(true, "تم ".$status." الطلب بنجاح", 1, "order", $updated_order));

        } catch (\Exception $exception) {
            return response()->json($this->result(false, "حدث خلل غير متوقع أثناء الإرسال، حاول في وقت آخر ", -2));
        }
    }

    function GetAddress($lat1, $lat2, $long1, $long2)
    {
        $url = "https://maps.googleapis.com/maps/api/distancematrix/json?origins=".$lat1.",".$long1."&destinations=".$lat2.",".$long2."&mode=driving&language=ar";
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_PROXYPORT, 3128);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
        $response = curl_exec($ch);
        curl_close($ch);
        $response_a = json_decode($response, true);
        /*        $dist = $response_a['rows'][0]['elements'][0]['distance']['text'];*/
        $address = $response_a['destination_addresses'][0];

        return /*array('distance' => $dist, 'time' => */$address/*)*/;
    }
}